/*
   FNT - Utility for manage settings of FNT-system.

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include "../include/global.h"
#include "../include/types.h"
#include "../include/error.h"
#include "../include/plugin.h"
#include "../include/sql.h"
#include "../include/sql-driver.h"
#include "../include/sql-config.h"
#include "../include/config-file.h"
#include "../include/main.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

static const char *import_file = NULL;
static const char *group_to_list = NULL;
static bool param_delete = false;
static bool param_list = false;

char *error_str;

bool OptionParce_list (const gchar *, const gchar *, gpointer, GError **);

static GOptionEntry command_line_entries[] = {
    {
        "list", 'L',
        G_OPTION_FLAG_OPTIONAL_ARG | G_OPTION_FLAG_NO_ARG,
        G_OPTION_ARG_CALLBACK, OptionParce_list,
         "List all parameters (only in [group] if present)", "[group]"
    },
    {"import", 'i', G_OPTION_FLAG_FILENAME, G_OPTION_ARG_FILENAME,
     &import_file, "Import settings from <ini-file>", "<ini-file>"},
    {"delete", 'd', 0, G_OPTION_ARG_NONE, &param_delete, "Delete parameter",
     NULL},
    {NULL}
};


/*** file scope functions **********************************************/

bool OptionParce_list (const gchar *option_name, const gchar *value, gpointer data, GError **error)
{
    param_list = true;
    group_to_list = value;
    return true;
}


int
deinit (fnt_sql_connect_t * sql_connect, int code)
{
    fnt_sql_disconnect (&sql_connect);
    fnt_deinit ();
    return code;
}


int
printParametersList (fnt_sql_connect_t * sql_connect, const char *group_name)
{
    GPtrArray *data;
    fnt_sql_config_data_t *one_element;
    int loop;
    char *grp_name = "";

    data = fnt_sql_config_getList (sql_connect, group_name);
    if (data == NULL)
    {
	error_str = fnt_error_getStackAsString ();
	fprintf (stderr, _("ERROR: Unable to get list of parameters\n%s"),
		 error_str);
	g_free (error_str);
	return deinit (sql_connect, 3);
    }
    for (loop = 0; loop < data->len; loop++)
    {
	one_element = g_ptr_array_index (data, loop);
	if (one_element->group == NULL)
	    continue;
	if (strcmp (one_element->group, grp_name))
	{
	    grp_name = one_element->group;
	    printf ("[%s]\n", grp_name);
	}
	printf ("\t%s = %s\n", one_element->param, one_element->value);
    }

    fnt_sql_config_freeList (&data);
    return deinit (sql_connect, 0);
}

int
deleteParameter (fnt_sql_connect_t * sql_connect, const char *name)
{
    if (name == NULL)
    {
	fprintf (stderr, _("ERROR: parameter absent\n"));
	return deinit (sql_connect, 3);
    }
    if (!fnt_sql_config_delete (sql_connect, name))
    {
	error_str = fnt_error_getStackAsString ();
	fprintf (stderr, _("ERROR: Unable delete parameter\n%s"), error_str);
	g_free (error_str);
	return deinit (sql_connect, 3);
    }
    return deinit (sql_connect, 0);
}

int
importSettings (fnt_sql_connect_t * sql_connect)
{
    fnt_config_file_t *config_file;
    fnt_sql_query_t *query;
    gchar **groups, **params;
    long long groups_last_inserted_id;
    char *escaped, *escaped2, *sql_string, *param_value;

    gsize groups_len, params_len, loop1, loop2;

    config_file = fnt_config_file_load (import_file);

    if (config_file == NULL)
    {
	error_str = fnt_error_getStackAsString ();
	fprintf (stderr, _("ERROR: Unable to load initial config file\n%s"),
		 error_str);
	g_free (error_str);
	return deinit (sql_connect, 2);
    }

    groups = g_key_file_get_groups (config_file->handle, &groups_len);
    if (groups == NULL)
    {
	fnt_config_file_deinit (&config_file);
	fprintf (stderr,
		 _("ERROR: Unable to get list of groups from config\n"));
	return deinit (sql_connect, 2);
    }

    query = fnt_sql_query (sql_connect, "DELETE FROM config_params");
    fnt_sql_queryClear (&query);
    query = fnt_sql_query (sql_connect, "DELETE FROM config_groups");
    fnt_sql_queryClear (&query);

    for (loop1 = 0; loop1 < groups_len; loop1++)
    {
	params =
	    g_key_file_get_keys (config_file->handle, groups[loop1],
				 &params_len, NULL);
	if (params == NULL)
	{
	    fprintf (stderr,
		     _
		     ("WARNING: Unable to get parameters list for group %s\n"),
		     groups[loop1]);
	    continue;
	}
	escaped = fnt_sql_escape (sql_connect, groups[loop1]);
	sql_string =
	    g_strdup_printf ("INSERT INTO config_groups (name) VALUES ('%s')",
			     escaped);
	g_free (escaped);
	query = fnt_sql_query (sql_connect, sql_string);
	g_free (sql_string);
	groups_last_inserted_id =
	    fnt_sql_lastInsertedId (query, "config_groups");
	fnt_sql_queryClear (&query);
	if (groups_last_inserted_id == 0)
	{
	    fprintf (stderr, _("WARNING: Unable to insert group name %s\n"),
		     groups[loop1]);
	    continue;
	}

	for (loop2 = 0; loop2 < params_len; loop2++)
	{
	    escaped =
		fnt_sql_config_getParameter (sql_connect, groups[loop1],
					     params[loop2]);
	    if (escaped != NULL)
	    {
		fprintf (stderr,
			 _("WARNING: Multiple declaration of %s.%s\n"),
			 groups[loop1], params[loop2]);
		g_free (escaped);
		continue;
	    }

	    param_value =
		g_key_file_get_value (config_file->handle, groups[loop1],
				      params[loop2], NULL);
	    if (param_value == NULL)
	    {
		fprintf (stderr,
			 _("WARNING: Unable to get value for %s.%s\n"),
			 groups[loop1], params[loop2]);
		continue;
	    }


	    escaped = fnt_sql_escape (sql_connect, params[loop2]);
	    escaped2 = fnt_sql_escape (sql_connect, param_value);
	    g_free (param_value);
	    param_value = g_strdup_printf ("%d", groups_last_inserted_id);
	    sql_string =
		g_strdup_printf
		("INSERT INTO config_params (config_group_id, name, value) \
VALUES (%s,'%s', '%s')", param_value, escaped, escaped2);
	    g_free (param_value);
	    g_free (escaped2);
	    g_free (escaped);
	    query = fnt_sql_query (sql_connect, sql_string);
	    g_free (sql_string);
	    fnt_sql_queryClear (&query);
	}
	g_strfreev (params);
    }

    g_strfreev (groups);
    fnt_config_file_deinit (&config_file);

    return deinit (sql_connect, 0);
}

int
changeOrDisplayParameter (fnt_sql_connect_t * sql_connect,
			  const char *param_string)
{
    fnt_sql_config_data_t *config_data;
    char *param_val;


    if (param_string == NULL)
    {
	fprintf (stderr, _("ERROR: parameter absent\n"));
	return deinit (sql_connect, 3);
    }

    config_data = fnt_sql_config_parceString (param_string);
    if (config_data == NULL)
    {
	fprintf (stderr, _("ERROR: unable parce string: %s\n"), param_string);
	return deinit (sql_connect, 3);
    }
    if (config_data->group == NULL)
    {
	fprintf (stderr,
		 _("ERROR: unable get group name from parce string: %s\n"),
		 param_string);
	return deinit (sql_connect, 3);
    }
    if (config_data->param == NULL)
    {
	fprintf (stderr,
		 _
		 ("ERROR: unable get parameter name from parce string: %s\n"),
		 param_string);
	return deinit (sql_connect, 3);
    }

    if (config_data->value == NULL)
    {
	param_val =
	    fnt_sql_config_getParameter (sql_connect, config_data->group,
					 config_data->param);
	g_free (config_data->group);
	g_free (config_data->param);
	g_free (config_data);

	if (param_val == NULL)
	    return deinit (sql_connect, 0);

	printf ("%s\n", param_val);
	g_free (param_val);

	return deinit (sql_connect, 0);
    }
    if (!fnt_sql_config_change (sql_connect, param_string))
    {
	error_str = fnt_error_getStackAsString ();
	fprintf (stderr, _("ERROR: unable set parameter: %s\n"), error_str);
	g_free (error_str);
	return deinit (sql_connect, 3);
    }
    return deinit (sql_connect, 0);
}

/*** public functions **************************************************/

int
main (int argc, char **argv)
{
    fnt_sql_connect_t *sql_connect = NULL;

    setlocale (LC_ALL, "");
    bindtextdomain (PACKAGE, LOCALEDIR);
    textdomain (PACKAGE);

    if (!fnt_parseCommandLineOptions (command_line_entries,
				      " - configure FNT system",
				      "Copyright (C) 2009 Free Software Foundation, Inc.\n\
Written by: Slava Zanko <slavazanko@gmail.com>\n\
This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you\n\
are welcome to redistribute it under certain conditions.\n\
\n\
" PACKAGE "-config -L [group]\n\
" PACKAGE "-config -i <ini-file>\n\
" PACKAGE "-config [options] group.name\n\
" PACKAGE "-config [options] group.name=value\n\
" PACKAGE "-config [options] -d group.name", NULL, NULL, &argc, &argv))
    {
        printf("%s\n",fnt_error_getStackAsString ());
        return 1;
    }

    if (!fnt_init ())
    {
	fprintf (stderr, _("ERROR: Unable initialize FNT library\n"));
	exit (2);
    }

    sql_connect = fnt_getConnectionFromIniFile (fnt_main_config_file, false);

    if (sql_connect == NULL)
    {
	error_str = fnt_error_getStackAsString ();
	fprintf (stderr, "%s", error_str);
	g_free (error_str);
	fnt_deinit ();
	exit (2);
    }

    if (param_list)
        return printParametersList (sql_connect, argv[optind]);

    if (param_delete)
	return deleteParameter (sql_connect, argv[optind]);

    if (import_file != NULL)
	return importSettings (sql_connect);

    return changeOrDisplayParameter (sql_connect, argv[optind]);
}
